from envisage.extension_point import ExtensionPoint
from envisage.plugin import Plugin
from traits.has_traits import on_trait_change
from traits.trait_types import List

from force_bdss.data_sources.i_data_source import (
    IDataSource)
from force_bdss.mco.i_multi_criteria_optimizers import IMultiCriteriaOptimizer


class CoreMCODriver(Plugin):
    """Main plugin that handles the execution of the MCO
    or the evaluation.
    """

    # Note: we are forced to declare these extensions points here instead
    # of the application object, and this is why we have to use this plugin.
    # It is a workaround to an envisage bug that does not find the extension
    # points if declared on the application.

    #: A List of the available Multi Criteria Optimizers.
    #: This will be populated by MCO plugins.
    multi_criteria_optimizers = ExtensionPoint(
        List(IMultiCriteriaOptimizer),
        id='force_bdss.multi_criteria_optimizers')

    #: A list of the available Key Performance Indicator calculators.
    #: It will be populated by plugins.
    data_sources = ExtensionPoint(
        List(IDataSource),
        id='force_bdss.data_sources')

    @on_trait_change("application:started")
    def application_started(self):
        workflow = self.application.workflow
        if self.application.evaluate:
            for requested_ds in workflow.data_sources:
                data_source = self._find_data_source_by_computes(requested_ds)
                if data_source:
                    data_source.run(self.application)
                else:
                    raise Exception("Requested data source {} but don't know "
                                    "to find it.".format(requested_ds))
        else:
            mco_name = workflow.multi_criteria_optimization.name
            mco = self._find_mco_by_name(mco_name)
            if mco:
                mco.run(self.application)
            else:
                raise Exception("Requested MCO {} but it's not available"
                                "to compute it.".format(mco_name))

    def _find_data_source_by_computes(self, computes):
        for ds in self.data_sources:
            if ds.computes == computes:
                return ds

        return None

    def _find_mco_by_name(self, name):
        for mco in self.multi_criteria_optimizers:
            if mco.name == name:
                return mco

        return None
